# Generated by Django 2.1 on 2018-08-28 22:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djcall', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='caller',
            name='signal_number',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
